@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -29,6 +29,7 @@ from __future__ import unicode_literals, absolute_import
 import os
 # import re
 import tempfile
+import warnings
 
 import six
 
@@ -69,31 +70,115 @@ class AppHandler(object):
         # app may not use the db layer, but if so we set the model
         try:
             self.model = config.get_model()
-        except:
+        except: # pragma: no cover
             pass
 
     def get_title(self, default=None):
+        """
+        Returns the configured title (name) of the app.
+
+        Default logic invokes
+        :meth:`rattail.config.RattailConfig.app_title()` to obtain the
+        title.
+
+        :param default: Value to be returned if there is no app title
+           configured.
+
+        :returns: Title for the app.
+        """
         return self.config.app_title(default=default)
 
     def get_timezone(self, key='default'):
+        """
+        Returns a configured time zone.
+
+        Default logic invokes :func:`rattail.time.timezone()` to
+        obtain the time zone object.
+
+        :param key: Unique key designating which time zone should be
+           returned.  Note that most apps have only one ("default"),
+           but may have others defined.
+        """
         from rattail.time import timezone
         return timezone(self.config, key)
 
     def localtime(self, *args, **kwargs):
+        """
+        Produce or convert a timestamp in the default time zone.
+
+        Default logic invokes :func:`rattail.time.localtime()` to
+        obtain the timestamp.  All args and kwargs are passed directly
+        to that function.
+
+        :returns: A :class:`python:datetime.datetime` object.  Usually
+           this will be timezone-aware but this will depend on the
+           args and kwargs you specify.
+        """
         from rattail.time import localtime
         return localtime(self.config, *args, **kwargs)
 
     def make_utc(self, *args, **kwargs):
+        """
+        Produce or convert a timestamp to UTC time zone.
+
+        Default logic invokes :func:`rattail.time.make_utc()` to
+        obtain the timestamp.  All args and kwargs are passed directly
+        to that function.
+
+        :returns: A :class:`python:datetime.datetime` object.  Usually
+           this will be timezone-naive but this will depend on the
+           args and kwargs you specify.
+        """
         from rattail.time import make_utc
         return make_utc(*args, **kwargs)
 
-    def load_entry_points(self, group, **kwargs):
+    def load_entry_points(self, group, **kwargs): # pragma: no cover
+        warnings.warn("method is deprecated, please use "
+                      "util.load_entry_points() function directly",
+                      DeprecationWarning)
         return load_entry_points(group, **kwargs)
 
+    def load_object(self, spec):
+        """
+        Import and/or load and return the object designated by the
+        given spec string.
+
+        Default logic invokes :func:`rattail.util.load_object()` to
+        obtain the object.
+
+        The syntax of the spec string is not unique to Rattail, but is
+        not a universal standard, so deserves a note here.  The spec
+        is basically just ``modulepath:objname`` where ``modulepath``
+        is the dotted name of the full module where the object
+        resides, and ``objname`` is the name of the object within that
+        module.  For instance, ``rattail.app:AppHandler`` would be the
+        spec for *this* class.
+
+        Note also that the use of the word "object" may be confusing,
+        as it does not signify an "instance" but rather an object in
+        the generic sense.  Most often a spec will either refer to a
+        class or function within the module, although any valid named
+        object is possible.
+
+        :param spec: String like ``modulepath:objname`` as described
+           above.
+
+        :returns: The object referred to by ``spec``.  If the module
+           could not be imported, or did not contain an object of the
+           given name, then an error will raise.
+        """
+        return load_object(spec)
+
     def get_active_stores(self, session, **kwargs):
         """
-        Returns the list of "active"
-        :class:`~rattail.db.model.stores:Store` records.
+        Returns the list of "active" stores.  A store is considered
+        active if it is *not* marked as archived.
+
+        :param session: Reference to current DB session.
+
+        :returns: Possibly-empty list of
+           :class:`~rattail.db.model.stores.Store` records which are
+           deemed active.
         """
         import sqlalchemy as sa
 
@@ -150,9 +235,15 @@ class AppHandler(object):
         if spec:
             return load_object(spec)(self.config)
 
-        raise NotImplementedError("cannot locate autocompleter for key: {}".format(key))
+        raise ValueError("cannot locate autocompleter for key: {}".format(key))
 
     def get_auth_handler(self, **kwargs):
+        """
+        Get the configured "auth" handler.
+
+        :returns: The :class:`~rattail.auth.AuthHandler` instance for
+           the app.
+        """
         if not hasattr(self, 'auth_handler'):
             spec = self.config.get('rattail', 'auth.handler',
                                    default='rattail.auth:AuthHandler')
@@ -160,42 +251,150 @@ class AppHandler(object):
             self.auth_handler = factory(self.config, **kwargs)
         return self.auth_handler
 
-    def get_batch_handler(self, key, **kwargs):
-        from rattail.batch import get_batch_handler
-        return get_batch_handler(self.config, key, **kwargs)
+    def get_batch_handler(self, key, default=None, error=True, **kwargs):
+        """
+        Get the configured batch handler of the given type.
+
+        :param key: Unique key designating which type of batch handler
+           is being requested.
+
+        :param default: Spec string to be used as the default, if no
+           handler is configured for the given batch type.  This spec
+           string must itself refer to a ``BatchHandler`` class.
+
+        :param error: Flag indicating whether an error should be
+           raised if no handler can be found.
+
+        :returns: A :class:`~rattail.batch.handlers.BatchHandler`
+           instance of the requested type.  If no such handler can be
+           found, and the ``error`` param is false, then ``None`` is
+           returned; otherwise an error will raise.
+        """
+        # spec is assumed to come from config/settings if present,
+        # otherwise caller-supplied default is assumed
+        spec = self.config.get('rattail.batch', '{}.handler'.format(key),
+                               default=default)
+
+        # TODO: this probably should go away?
+        # if none of the above gave us a spec, check for common 'importer' type
+        if not spec and key == 'importer':
+            spec = 'rattail.batch.importer:ImporterBatchHandler'
+
+        if spec:
+            Handler = self.load_object(spec)
+            return Handler(self.config)
+
+        if error:
+            raise ValueError("handler spec not found for batch "
+                             "type: {}".format(key))
 
     def get_board_handler(self, **kwargs):
+        """
+        Get the configured "board" handler.
+
+        :returns: The :class:`~rattail.board.BoardHandler` instance
+           for the app.
+        """
         if not hasattr(self, 'board_handler'):
             from rattail.board import get_board_handler
             self.board_handler = get_board_handler(self.config, **kwargs)
         return self.board_handler
 
+    def get_bounce_handler(self, key, **kwargs):
+        """
+        Get the configured email bounce handler of the given type.
+
+        :param key: Unique key designating which type of bounce
+           handler is being requested.
+
+        :returns: A :class:`~rattail.bouncer.handler.BounceHandler`
+           instance of the requested type.  If no such handler can be
+           found, an error will raise.
+        """
+        if not hasattr(self, 'bounce_handlers'):
+            self.bounce_handlers = {}
+
+        if key not in self.bounce_handlers:
+            spec = self.config.get('rattail.bouncer',
+                                   '{}.handler'.format(key))
+            if not spec and key == 'default':
+                spec = 'rattail.bouncer:BounceHandler'
+            if not spec:
+                raise ValueError("bounce handler spec not found for "
+                                 "type: {}".format(key))
+
+            Handler = self.load_object(spec)
+            self.bounce_handlers[key] = Handler(self.config, key)
+
+        return self.bounce_handlers[key]
+
     def get_clientele_handler(self, **kwargs):
+        """
+        Get the configured "clientele" handler.
+
+        :returns: The :class:`~rattail.clientele.ClienteleHandler`
+           instance for the app.
+        """
         if not hasattr(self, 'clientele_handler'):
             from rattail.clientele import get_clientele_handler
             self.clientele_handler = get_clientele_handler(self.config, **kwargs)
         return self.clientele_handler
 
+    def get_custorder_handler(self, **kwargs):
+        """
+        Get the configured "customer order" handler.
+
+        :returns: The :class:`~rattail.custorders.CustomerOrderHandler`
+           instance for the app.
+        """
+        if not hasattr(self, 'custorder_handler'):
+            spec = self.config.get('rattail', 'custorders.handler',
+                                   default='rattail.custorders:CustomerOrderHandler')
+            Handler = self.load_object(spec)
+            self.custorder_handler = Handler(self.config)
+        return self.custorder_handler
+
     def get_employment_handler(self, **kwargs):
+        """
+        Get the configured "employment" handler.
+
+        :returns: The :class:`~rattail.employment.EmploymentHandler`
+           instance for the app.
+        """
         if not hasattr(self, 'employment_handler'):
             from rattail.employment import get_employment_handler
             self.employment_handler = get_employment_handler(self.config, **kwargs)
         return self.employment_handler
 
     def get_feature_handler(self, **kwargs):
+        """
+        Get the configured "feature" handler.
+
+        :returns: The :class:`~rattail.features.handlers.FeatureHandler`
+           instance for the app.
+        """
         if not hasattr(self, 'feature_handler'):
             from rattail.features import FeatureHandler
             self.feature_handler = FeatureHandler(self.config, **kwargs)
         return self.feature_handler
 
     def get_email_handler(self, **kwargs):
+        """
+        Get the configured "email" handler.
+
+        :returns: The :class:`~rattail.mail.EmailHandler` instance for
+           the app.
+        """
         if not hasattr(self, 'email_handler'):
             from rattail.mail import get_email_handler
             self.email_handler = get_email_handler(self.config, **kwargs)
         return self.email_handler
 
-    # TODO: is it helpful to expose this? or more confusing?
-    get_mail_handler = get_email_handler
+    def get_mail_handler(self, **kwargs): # pragma: no cover
+        warnings.warn("method is deprecated, please use "
+                      "AppHandler.get_email_handler() instead",
+                      DeprecationWarning)
+        return self.get_email_handler(**kwargs)
 
     def get_all_import_handlers(self, ignore_errors=True, sort=False,
                                 **kwargs):
@@ -223,8 +422,8 @@ class AppHandler(object):
         :returns: List of all registered Import/Export Handler classes.
         """
         # first load all "registered" Handler classes
-        Handlers = self.load_entry_points('rattail.importing',
-                                          ignore_errors=ignore_errors)
+        Handlers = load_entry_points('rattail.importing',
+                                     ignore_errors=ignore_errors)
 
         # organize registered classes by spec
         specs = {}
@@ -262,7 +461,7 @@ class AppHandler(object):
         *Handler classes*.
 
         If you have a type key and just need its designated handler,
-        see :meth:`get_designated_import_handler()`.
+        see :meth:`get_import_handler()`.
 
         See also :meth:`get_all_import_handlers()` if you need all
         registered Handler classes.
@@ -293,9 +492,6 @@ class AppHandler(object):
             if len(group) == 1:
                 return group[0]
 
-            raise RuntimeError("multiple ({}) handlers available but none designated for: {}".format(
-                len(group), group[0].get_key()))
-
         designated = []
         for key, group in six.iteritems(grouped):
             Handler = find_designated(key, group)
@@ -312,7 +508,7 @@ class AppHandler(object):
 
         return designated
 
-    def get_designated_import_handler(self, key, require=False, **kwargs):
+    def get_import_handler(self, key, require=False, **kwargs):
         """
         Return the designated import/export handler instance, per the
         given handler type key.
@@ -332,25 +528,41 @@ class AppHandler(object):
            case error is raised.
         """
         # first try to fetch the handler per designated spec
-        spec = self.get_designated_import_handler_spec(key, require=require,
-                                                       **kwargs)
+        spec = self.get_designated_import_handler_spec(key, **kwargs)
         if spec:
-            Handler = load_object(spec)
+            Handler = self.load_object(spec)
             return Handler(self.config)
 
         # nothing was designated, so leverage logic which already
         # sorts out which handler is "designated" for given key
-        designated = self.get_designated_import_handlers(
-            ignore_errors=not require)
+        designated = self.get_designated_import_handlers()
         for handler in designated:
             if handler.get_key() == key:
                 return handler
 
         if require:
-            raise RuntimeError("Cannot locate designated handler for key: {}".format(
-                key))
+            raise ValueError("Cannot locate handler for key: {}".format(key))
+
+    def get_designated_import_handler(self, *args, **kwargs): # pragma: no cover
+        warnings.warn("method is deprecated, please use "
+                      "AppHandler.get_import_handler() instead",
+                      DeprecationWarning)
+        return self.get_import_handler(*args, **kwargs)
 
     def get_designated_import_handler_spec(self, key, require=False, **kwargs):
+        """
+        Return the designated import handler "spec" string for the
+        given type key.
+
+        :param key: Unique key indicating the type of import handler.
+
+        :require: Flag indicating whether an error should be raised if no
+           handler is found.
+
+        :returns: Spec string for the designated handler.  If none is
+           found, then ``None`` is returned *unless* the ``require``
+           param is true, in which case an error is raised.
+        """
         spec = self.config.get('rattail.importing',
                                '{}.handler'.format(key))
         if spec:
@@ -372,13 +584,16 @@ class AppHandler(object):
             return spec
 
         if require:
-            raise RuntimeError("Cannot locate handler spec for key: {}".format(key))
+            raise ValueError("Cannot locate handler spec for key: {}".format(key))
 
     def get_membership_handler(self, **kwargs):
         """
-        Returns a reference to the configured Membership Handler.
+        Get the configured "membership" handler.
 
         See also :doc:`rattail-manual:base/handlers/other/membership`.
+
+        :returns: The :class:`~rattail.membership.MembershipHandler`
+           instance for the app.
         """
         if not hasattr(self, 'membership_handler'):
             spec = self.config.get('rattail', 'membership.handler',
@@ -389,9 +604,12 @@ class AppHandler(object):
 
     def get_people_handler(self, **kwargs):
         """
-        Returns a reference to the configured People Handler.
+        Get the configured "people" handler.
 
         See also :doc:`rattail-manual:base/handlers/other/people`.
+
+        :returns: The :class:`~rattail.people.PeopleHandler` instance
+           for the app.
         """
         if not hasattr(self, 'people_handler'):
             spec = self.config.get('rattail', 'people.handler',
@@ -401,42 +619,97 @@ class AppHandler(object):
         return self.people_handler
 
     def get_products_handler(self, **kwargs):
+        """
+        Get the configured "products" handler.
+
+        :returns: The :class:`~rattail.products.ProductsHandler`
+           instance for the app.
+        """
         if not hasattr(self, 'products_handler'):
             from rattail.products import get_products_handler
             self.products_handler = get_products_handler(self.config, **kwargs)
         return self.products_handler
 
     def get_report_handler(self, **kwargs):
+        """
+        Get the configured "reports" handler.
+
+        :returns: The :class:`~rattail.reporting.handlers.ReportHandler`
+           instance for the app.
+        """
         if not hasattr(self, 'report_handler'):
             from rattail.reporting import get_report_handler
             self.report_handler = get_report_handler(self.config, **kwargs)
         return self.report_handler
 
     def get_problem_report_handler(self, **kwargs):
+        """
+        Get the configured "problem reports" handler.
+
+        :returns: The :class:`~rattail.problems.handlers.ProblemReportHandler`
+           instance for the app.
+        """
         if not hasattr(self, 'problem_report_handler'):
             from rattail.problems import get_problem_report_handler
             self.problem_report_handler = get_problem_report_handler(
                 self.config, **kwargs)
         return self.problem_report_handler
 
+    def get_trainwreck_handler(self, **kwargs):
+        """
+        Get the configured "trainwreck" handler.
+
+        :returns: The :class:`~rattail.trainwreck.handler.TrainwreckHandler`
+           instance for the app.
+        """
+        if not hasattr(self, 'trainwreck_handler'):
+            spec = self.config.get('trainwreck', 'handler',
+                                   default='rattail.trainwreck.handler:TrainwreckHandler')
+            Handler = self.load_object(spec)
+            self.trainwreck_handler = Handler(self.config)
+        return self.trainwreck_handler
+
     def progress_loop(self, *args, **kwargs):
+        """
+        Run a given function for a given sequence, and optionally show
+        a progress indicator.
+
+        Default logic invokes the :func:`rattail.util.progress_loop()`
+        function; see that for more details.
+        """
         return progress_loop(*args, **kwargs)
 
     def get_session(self, obj):
         """
         Returns the SQLAlchemy session with which the given object is
         associated.  Simple convenience wrapper around
-        ``sqlalchemy.orm.object_session()``.
+        :func:`sqlalchemy:sqlalchemy.orm.object_session()`.
         """
         from sqlalchemy import orm
 
         return orm.object_session(obj)
 
-    def make_session(self, **kwargs):
+    def make_session(self, user=None, **kwargs):
         """
         Creates and returns a new SQLAlchemy session for the Rattail DB.
+
+        :param user: A "user-ish" object which should be considered
+           responsible for changes made during the session.  Can be
+           either a :class:`~rattail.db.model.users.User` object, or
+           just a (string) username.  If none is specified then the
+           config will be consulted for a default.
+
+        :returns: A :class:`rattail.db.Session` instance.
         """
         from rattail.db import Session
+
+        # always try to set default continuum user if possible
+        if 'continuum_user' not in kwargs:
+            if not user:
+                user = self.config.get('rattail', 'runas.default')
+            if user:
+                kwargs['continuum_user'] = user
+
         return Session(**kwargs)
 
     def cache_model(self, session, model, **kwargs):
@@ -450,8 +723,15 @@ class AppHandler(object):
 
     def make_temp_dir(self, **kwargs):
         """
-        Create a temporary directory.  This is mostly a convenience wrapper
-        around the built-in ``tempfile.mkdtemp()``.
+        Create a temporary directory.  This is mostly a convenience
+        wrapper around the built-in :func:`python:tempfile.mkdtemp()`.
+        However by default it will attempt to place the temp folder
+        underneath the configured "workdir", e.g.:
+
+        .. code-block:: ini
+
+           [rattail]
+           workdir = /srv/envs/poser/app/work
         """
         if 'dir' not in kwargs:
             workdir = self.config.workdir(require=False)
@@ -464,8 +744,15 @@ class AppHandler(object):
 
     def make_temp_file(self, **kwargs):
         """
-        Reserve a temporary filename.  This is mostly a convenience wrapper
-        around the built-in ``tempfile.mkstemp()``.
+        Reserve a temporary filename.  This is mostly a convenience
+        wrapper around the built-in :func:`python:tempfile.mkstemp()`.
+        However by default it will attempt to place the temp file
+        underneath the configured "workdir", e.g.:
+
+        .. code-block:: ini
+
+           [rattail]
+           workdir = /srv/envs/poser/app/work
         """
         if 'dir' not in kwargs:
             workdir = self.config.workdir(require=False)
@@ -476,10 +763,22 @@ class AppHandler(object):
                 kwargs['dir'] = tmpdir
         return temp_path(**kwargs)
 
-    def normalize_phone_number(self, number):
+    def normalize_phone_number(self, number, **kwargs):
         """
         Normalize the given phone number, to a "common" format that
-        can be more easily worked with for sync logic etc.
+        can be more easily worked with for sync logic etc.  In
+        practice this usually just means stripping all non-digit
+        characters from the string.  The idea is that phone number
+        data from any system can be "normalized" and thereby compared
+        directly to see if they differ etc.
+
+        Default logic will invoke
+        :func:`rattail.db.util.normalize_phone_number()`.
+
+        :param number: Raw phone number string e.g. as found in some
+           data source.
+
+        :returns: Normalized string.
         """
         from rattail.db.util import normalize_phone_number
 
@@ -487,9 +786,15 @@ class AppHandler(object):
 
     def phone_number_is_invalid(self, number):
         """
-        This method should validate the given phone number string, and if the
-        number is *not* considered valid, this method should return the reason
-        as string.  If the number is valid, returns ``None``.
+        This method should validate the given phone number string, and
+        if the number is *not* considered valid, this method should
+        return the reason.
+
+        :param number: Raw phone number string e.g. as found in some
+           data source.
+
+        :returns: String describing reason the number is invalid, or
+           ``None`` if the number is deemed valid.
         """
         # strip non-numeric chars, and make sure we have 10 left
         normal = self.normalize_phone_number(number)
@@ -498,16 +803,63 @@ class AppHandler(object):
 
     def format_phone_number(self, number):
         """
-        Returns a "properly formatted" string based on the given phone number.
+        Returns a "properly formatted" string based on the given phone
+        number.
+
+        Default logic invokes
+        :func:`rattail.db.util.format_phone_number()`.
+
+        :param number: Raw phone number string e.g. as found in some
+           data source.
+
+        :returns: Formatted phone number string.
         """
         from rattail.db.util import format_phone_number
 
         return format_phone_number(number)
 
+    def make_gpc(self, value, **kwargs):
+        """
+        Make and return a :class:`~rattail.gpc.GPC` instance from the
+        given value.
+
+        Default logic will invoke
+        :meth:`~rattail.products.ProductsHandler.make_gpc()` of the
+        products handler; see also :meth:`get_products_handler()`.
+        """
+        products_handler = self.get_products_handler()
+        return products_handler.make_gpc(value, **kwargs)
+
+    def render_gpc(self, value, **kwargs):
+        """
+        Returns a human-friendly display string for the given GPC
+        value.
+
+        :param value: A :class:`~rattail.gpc.GPC` instance.
+
+        :returns: Display string for the GPC, or ``None`` if the value
+           provided is not a GPC.
+        """
+        if value:
+            return value.pretty()
+
+    def render_upc(self, value, **kwargs): # pragma: no cover
+        warnings.warn("method is deprecated, please use "
+                      "render_gpc() method instead",
+                      DeprecationWarning)
+        return self.render_gpc(value, **kwargs)
+
     def render_currency(self, value, scale=2, **kwargs):
         """
-        Must return a human-friendly display string for the given currency
-        value, e.g. ``Decimal('4.20')`` becomes ``"$4.20"``.
+        Must return a human-friendly display string for the given
+        currency value, e.g. ``Decimal('4.20')`` becomes ``"$4.20"``.
+
+        :param value: Either a :class:`python:decimal.Decimal` or
+           :class:`python:float` value.
+
+        :param scale: Number of decimal digits to be displayed.
+
+        :returns: Display string for the value.
         """
         if value is not None:
             if value < 0:
@@ -520,13 +872,32 @@ class AppHandler(object):
         """
         Return a human-friendly display string for the given quantity
         value, e.g. ``1.000`` becomes ``"1"``.
+
+        :param value: The quantity to be rendered.
+
+        :returns: Display string for the quantity.
         """
         return pretty_quantity(value, **kwargs)
 
     def render_cases_units(self, cases, units):
         """
         Render a human-friendly string showing the given number of
-        cases and/or units.
+        cases and/or units.  For instance::
+
+           >>> app.render_cases_units(1, None)
+           '1 case'
+
+           >>> app.render_cases_units(None, 1)
+           '1 unit'
+
+           >>> app.render_cases_units(3, 2)
+           '3 cases + 2 units'
+
+        :param cases: Number of cases (can be zero or ``None``).
+
+        :param units: Number of units (can be zero or ``None``).
+
+        :returns: Display string for the given values.
         """
         if cases is not None:
             label = "case" if abs(cases) == 1 else "cases"
@@ -543,16 +914,18 @@ class AppHandler(object):
 
     def render_date(self, value, **kwargs):
         """
-        Must return a human-friendly display string for the given ``date``
-        object.
+        Return a human-friendly display string for the given date.
+
+        :param value: A :class:`python:datetime.date` instance.
+
+        :returns: Display string for the date.
         """
         if value is not None:
             return value.strftime('%Y-%m-%d')
 
     def render_datetime(self, value, **kwargs):
         """
-        Must return a human-friendly display string for the given ``datetime``
-        object.
+        Return a human-friendly display string for the given datetime.
         """
         if value is not None:
             return value.strftime('%Y-%m-%d %I:%M:%S %p')
@@ -580,36 +953,33 @@ class GenericHandler(object):
         self.model = self.config.get_model()
         self.app = self.config.get_app()
 
-    def progress_loop(self, *args, **kwargs):
+    def progress_loop(self, *args, **kwargs): # pragma: no cover
+        warnings.warn("method is deprecated, please use "
+                      "AppHandler.progress_loop() method instead",
+                      DeprecationWarning)
         return self.app.progress_loop(*args, **kwargs)
 
-    def get_session(self, obj):
-        """
-        Convenience wrapper around :meth:`AppHandler.get_session()`.
-        """
+    def get_session(self, obj): # pragma: no cover
+        warnings.warn("method is deprecated, please use "
+                      "AppHandler.get_session() method instead",
+                      DeprecationWarning)
         return self.app.get_session(obj)
 
-    def make_session(self):
-        """
-        Convenience wrapper around :meth:`AppHandler.make_session()`.
-        """
+    def make_session(self): # pragma: no cover
+        warnings.warn("method is deprecated, please use "
+                      "AppHandler.make_session() method instead",
+                      DeprecationWarning)
         return self.app.make_session()
 
-    def cache_model(self, session, model, **kwargs):
-        """
-        Convenience method which invokes :func:`rattail.db.cache.cache_model()`
-        with the given model and keyword arguments.
-        """
+    def cache_model(self, session, model, **kwargs): # pragma: no cover
+        warnings.warn("method is deprecated, please use "
+                      "AppHandler.cache_model() method instead",
+                      DeprecationWarning)
         return self.app.cache_model(session, model, **kwargs)
 
 
-def make_app(config, **kwargs):
-    """
-    Create and return the configured :class:`AppHandler` instance.
-    """
-    spec = config.get('rattail', 'app.handler')
-    if spec:
-        factory = load_object(spec)
-    else:
-        factory = AppHandler
-    return factory(config, **kwargs)
+def make_app(config, **kwargs): # pragma: no cover
+    warnings.warn("function is deprecated, please use "
+                  "RattailConfig.get_app() method instead",
+                  DeprecationWarning)
+    return config.get_app()
