@@ -21,7 +21,9 @@
 #
 ################################################################################
 """
-Handler for "customer order" batches
+Customer Order Batch Handler
+
+Please note this is different from the Customer Order Handler.
 """
 
 from __future__ import unicode_literals, absolute_import, division
@@ -31,7 +33,6 @@ import decimal
 
 import six
 import sqlalchemy as sa
-from sqlalchemy import orm
 
 from rattail.db import model
 from rattail.batch import BatchHandler
@@ -84,16 +85,26 @@ class CustomerOrderBatchHandler(BatchHandler):
         """
         return self.config.getbool('rattail.custorders',
                                    'new_orders.allow_contact_info_choice',
-                                   default=True)
+                                   default=False)
 
-    def should_restrict_contact_info(self):
+    def allow_contact_info_creation(self):
         """
-        Returns a boolean indicating whether contact info should be
-        "restricted" - i.e. user can only choose from existing contact
-        info and cannot override by e.g. entering a new phone number.
+        Returns a boolean indicating whether the user is allowed to
+        enter *new* contact info for the customer.  This setting
+        should only be honored if :meth:`allow_contact_info_choice()`
+        also returns true.
         """
         return self.config.getbool('rattail.custorders',
-                                   'new_orders.restrict_contact_info',
+                                   'new_orders.allow_contact_info_create',
+                                   default=False)
+
+    def allow_unknown_product(self):
+        """
+        Returns a boolean indicating whether "unknown" products are
+        allowed on new orders.
+        """
+        return self.config.getbool('rattail.custorders',
+                                   'allow_unknown_product',
                                    default=False)
 
     def product_price_may_be_questionable(self):
@@ -460,12 +471,15 @@ class CustomerOrderBatchHandler(BatchHandler):
         info = {
             'uuid': product.uuid,
             'upc': six.text_type(product.upc),
+            'item_id': product.item_id,
+            'scancode': product.scancode,
             'upc_pretty': product.upc.pretty(),
             'brand_name': product.brand.name if product.brand else None,
             'description': product.description,
             'size': product.size,
             'full_description': product.full_description,
             'case_quantity': self.app.render_quantity(self.get_case_size_for_product(product)),
+            'unit_price': float(product.regular_price.price) if product.regular_price and product.regular_price.price is not None else None,
             'unit_price_display': products.render_price(product.regular_price),
             'department_name': product.department.name if product.department else None,
             'vendor_name': vendor.name if vendor else None,
@@ -510,6 +524,28 @@ class CustomerOrderBatchHandler(BatchHandler):
 
         return info
 
+    def uom_choices_for_row(self, row):
+        """
+        Return a list of UOM choices for the given batch row.
+        """
+        if row.product:
+            return self.uom_choices_for_product(row.product)
+
+        choices = []
+
+        # Each
+        unit_name = self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_EACH]
+        choices.append({'key': self.enum.UNIT_OF_MEASURE_EACH,
+                        'value': unit_name})
+
+        # Case
+        case_text = self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_CASE]
+        if case_text:
+            choices.append({'key': self.enum.UNIT_OF_MEASURE_CASE,
+                            'value': case_text})
+
+        return choices
+
     def uom_choices_for_product(self, product):
         """
         Return a list of UOM choices for the given product.
@@ -575,35 +611,135 @@ class CustomerOrderBatchHandler(BatchHandler):
         self.add_row(batch, row)
         return row
 
+    def add_pending_product(self, batch, pending_info,
+                            order_quantity, order_uom,
+                            **kwargs):
+        """
+        Add a new row to the batch, for the given "pending" product
+        and order quantity.
+
+        :param batch: Reference to the current order batch.
+
+        :param pending_info: Dict of information about a new pending product.
+
+        :param order_quantity: Quantity of the item to be added to the order.
+
+        :param order_uom: Unit of measure for the order quantity.
+        """
+        session = self.app.get_session(batch)
+
+        # make a new pending product
+        pending_product = self.make_pending_product(**pending_info)
+        session.add(pending_product)
+        session.flush()
+
+        # TODO: seems like make_pending_product() should do this?
+        if pending_product.department and not pending_product.department_name:
+            pending_product.department_name = pending_product.department.name
+
+        # make a new row, w/ pending product
+        row = self.make_row()
+        row.pending_product = pending_product
+        row.order_quantity = order_quantity
+        row.order_uom = order_uom
+        self.add_row(batch, row)
+        return row
+
+    def make_pending_product(self, **kwargs):
+        products_handler = self.app.get_products_handler()
+        if 'status_code' not in kwargs:
+            kwargs['status_code'] = self.enum.PENDING_PRODUCT_STATUS_PENDING
+        return products_handler.make_pending_product(**kwargs)
+
+    def update_pending_product(self, row, data):
+        """
+        Update the pending product data for the given batch row.
+        """
+        # create pending product if needed
+        pending = row.pending_product
+        if not pending:
+            pending = self.make_pending_product(**data)
+
+        simple_fields = [
+            'scancode',
+            'item_id',
+            'item_type',
+            'department_name',
+            'department_uuid',
+            'brand_name',
+            'brand_uuid',
+            'description',
+            'size',
+            'vendor_name',
+            'vendor_uuid',
+            'vendor_item_code',
+            'special_order',
+            'notes',
+        ]
+
+        decimal_fields = [
+            'unit_cost',
+            'case_size',
+            'case_cost',
+            'regular_price_amount',
+        ]
+
+        # update pending product info
+        if 'upc' in data:
+            pending.upc = self.app.make_gpc(data['upc']) if data['upc'] else None
+        for field in simple_fields:
+            if field in data:
+                setattr(pending, field, data[field])
+        for field in decimal_fields:
+            if field in data:
+                value = data[field]
+                value = decimal.Decimal(value) if value is not None else None
+                setattr(pending, field, value)
+
+        # refresh the row per new pending product info
+        self.refresh_row(row)
+
     def refresh_row(self, row):
         if not row.product:
             if row.item_entry:
-                session = orm.object_session(row)
-                # TODO: should do more than just query for uuid here
-                product = session.query(model.Product).get(row.item_entry)
+                session = self.app.get_session(row)
+                product = self.locate_product_for_entry(session, row.item_entry)
                 if product:
                     row.product = product
-            if not row.product:
+            if not row.product and not row.pending_product:
                 row.status_code = row.STATUS_PRODUCT_NOT_FOUND
                 return
 
         product = row.product
-        row.product_upc = product.upc
-        row.product_brand = six.text_type(product.brand or "")
-        row.product_description = product.description
-        row.product_size = product.size
-        row.product_weighed = product.weighed
-        row.case_quantity = self.get_case_size_for_product(product)
+        if product:
 
-        department = product.department
-        row.department_number = department.number if department else None
-        row.department_name = department.name if department else None
+            row.product_upc = product.upc
+            row.product_item_id = product.item_id
+            row.product_scancode = product.scancode
+            row.product_brand = six.text_type(product.brand or "")
+            row.product_description = product.description
+            row.product_size = product.size
+            row.product_weighed = product.weighed
+            row.case_quantity = self.get_case_size_for_product(product)
 
-        cost = product.cost
-        row.product_unit_cost = cost.unit_cost if cost else None
+            department = product.department
+            row.department_number = department.number if department else None
+            row.department_name = department.name if department else None
 
-        regprice = product.regular_price
-        row.unit_price = regprice.price if regprice else None
+            cost = product.cost
+            row.product_unit_cost = cost.unit_cost if cost else None
+
+            regprice = product.regular_price
+            row.unit_regular_price = regprice.price if regprice else None
+
+            curprice = product.current_price
+            row.unit_sale_price = curprice.price if curprice else None
+            row.sale_ends = curprice.ends if curprice else None
+
+            row.unit_price = row.unit_sale_price or row.unit_regular_price
+
+        else:
+            self.refresh_row_from_pending_product(row)
 
         # we need to know if total price is updated
         old_total = row.total_price
@@ -625,7 +761,47 @@ class CustomerOrderBatchHandler(BatchHandler):
                                  + (row.total_price or 0)
                                  - (old_total or 0))
 
-        row.status_code = row.STATUS_OK
+        if not row.product and row.pending_product:
+            row.status_code = row.STATUS_PENDING_PRODUCT
+        else:
+            row.status_code = row.STATUS_OK
+
+    def refresh_row_from_pending_product(self, row):
+        """
+        Refresh basic row attributes from its pending product.
+        """
+        pending = row.pending_product
+        if not pending:
+            return
+
+        row.product_upc = pending.upc
+        row.product_item_id = pending.item_id
+        row.product_scancode = pending.scancode
+        row.product_brand = six.text_type(pending.brand or pending.brand_name or '')
+        row.product_description = pending.description
+        row.product_size = pending.size
+        # TODO: is this even important?  pending does not have it
+        # row.product_weighed = pending.weighed
+        row.case_quantity = pending.case_size
+
+        if pending.department:
+            row.department = pending.department
+            row.department_number = pending.department.number
+            row.department_name = pending.department.name
+        elif pending.department_name:
+            row.department = None
+            row.department_number = None
+            row.department_name = pending.department_name
+        else:
+            row.department = None
+            row.department_number = None
+            row.department_name = None
+
+        row.product_unit_cost = pending.unit_cost
+        row.unit_regular_price = pending.regular_price_amount
+        row.unit_price = row.unit_regular_price
+        row.unit_sale_price = None
+        row.sale_ends = None
 
     def remove_row(self, row):
         batch = row.batch
@@ -638,16 +814,46 @@ class CustomerOrderBatchHandler(BatchHandler):
 
         self.refresh_batch_status(batch)
 
+    def delete_extra_data(self, batch, progress=None, **kwargs):
+
+        # do the normal stuff (e.g. delete input files)
+        super(CustomerOrderBatchHandler, self).delete_extra_data(
+            batch, progress=progress, **kwargs)
+
+        session = self.app.get_session(batch)
+
+        # delete pending customer if present
+        pending = batch.pending_customer
+        if pending:
+            batch.pending_customer = None
+            session.delete(pending)
+
+        # delete any pending products if present
+        for row in batch.data_rows:
+            pending = row.pending_product
+            if pending:
+                row.pending_product = None
+                session.delete(pending)
+
     def execute(self, batch, user=None, progress=None, **kwargs):
         """
         Default behavior here will create and return a new rattail
         Customer Order.  It also may "add contact info" e.g. to the
         customer record.  Override as needed.
         """
+        rows = batch.active_rows()
         order = self.make_new_order(batch, user=user, progress=progress, **kwargs)
         self.update_contact_info(batch, user)
+        self.mark_pending_things_ready(batch, rows)
         return order
 
+    def mark_pending_things_ready(self, batch, rows):
+        if batch.pending_customer:
+            batch.pending_customer.status = self.enum.PENDING_CUSTOMER_STATUS_READY
+        for row in rows:
+            if row.pending_product:
+                row.pending_product.status = self.enum.PENDING_PRODUCT_STATUS_READY
+
     def update_contact_info(self, batch, user, **kwargs):
         """
         Update contact info from the batch, onto the customer record.
@@ -711,8 +917,13 @@ class CustomerOrderBatchHandler(BatchHandler):
         for field in batch_fields:
             setattr(order, field, getattr(batch, field))
 
+        session = self.app.get_session(batch)
+        session.add(order)
+        session.flush()
+
         row_fields = [
             'product',
+            'pending_product_uuid',
             'product_upc',
             'product_brand',
             'product_description',
@@ -747,10 +958,7 @@ class CustomerOrderBatchHandler(BatchHandler):
         self.progress_loop(convert, batch.active_rows(), progress,
                            message="Converting batch rows to order items")
 
-        session = orm.object_session(batch)
-        session.add(order)
         session.flush()
-
         return order
 
     def set_initial_item_status(self, item, user, **kwargs):
