@@ -57,28 +57,18 @@ class ProductBatchHandler(BatchHandler):
                 return
 
             session = orm.object_session(row)
-            row.product = self.locate_product_for_entry(session, row.item_entry)
+            # nb. we prefer a GPC lookup if one is present
+            row.product = self.locate_product_for_entry(session,
+                                                        row.upc or row.item_entry)
             if not row.product:
                 row.status_code = row.STATUS_PRODUCT_NOT_FOUND
                 return
 
+        self.refresh_product_basics(row)
         product = row.product
-        row.upc = product.upc
-        row.item_id = product.item_id
-        row.brand_name = product.brand.name if product.brand else None
-        row.description = product.description
-        row.size = product.size
-
-        dept = product.department
-        row.department = dept
-        row.department_number = dept.number if dept else None
-        row.department_name = dept.name if dept else None
-
-        subdept = product.subdepartment
-        row.subdepartment = subdept
-        row.subdepartment_number = subdept.number if subdept else None
-        row.subdepartment_name = subdept.name if subdept else None
 
+        row.department = product.department
+        row.subdepartment = product.subdepartment
         row.category = product.category
         row.family = product.family
         row.reportcode = product.report_code
