@@ -32,7 +32,7 @@ from sqlalchemy import orm
 
 from rattail.db import Session, model
 from rattail.db.util import make_topo_sortkey
-from rattail.datasync import DataSyncWatcher, DataSyncConsumer, NewDataSyncImportConsumer
+from rattail.datasync import DataSyncWatcher, DataSyncConsumer, DataSyncImportConsumer
 from rattail.config import parse_list
 
 
@@ -52,9 +52,9 @@ class RattailWatcher(DataSyncWatcher):
 
     def get_changes(self, lastrun):
         """
-        Checks the :class:`rattail.db.model:Change` table in a Rattail
-        database, to see if there are any pending changes for the datasync
-        daemon.
+        Checks the :class:`~rattail.db.model.core.Change` table in a
+        Rattail database, to see if there are any pending changes for
+        the datasync daemon.
         """
         session = Session(bind=self.engine)
         changes = session.query(model.Change).all()
@@ -346,7 +346,7 @@ class RattailConsumer(DataSyncConsumer):
             session.delete(assignment)
 
 
-class FromRattailToRattailBase(NewDataSyncImportConsumer):
+class FromRattailToRattailBase(DataSyncImportConsumer):
     """
     Base class for Rattail -> Rattail datasync consumers
     """
