@@ -990,6 +990,7 @@ class PendingProduct(Base):
         sa.ForeignKeyConstraint(['user_uuid'], ['user.uuid'], name='pending_product_fk_user'),
         sa.ForeignKeyConstraint(['department_uuid'], ['department.uuid'], name='pending_product_fk_department'),
         sa.ForeignKeyConstraint(['brand_uuid'], ['brand.uuid'], name='pending_product_fk_brand'),
+        sa.ForeignKeyConstraint(['vendor_uuid'], ['vendor.uuid'], name='pending_product_fk_vendor'),
     )
 
     uuid = uuid_column()
@@ -1023,7 +1024,15 @@ class PendingProduct(Base):
 
     description = sa.Column(sa.String(length=255), nullable=True)
     size = sa.Column(sa.String(length=30), nullable=True)
+
+    vendor_name = sa.Column(sa.String(length=50), nullable=True)
+    vendor_uuid = sa.Column(sa.String(length=32), nullable=True)
+    vendor = orm.relationship(Vendor)
+
+    vendor_item_code = sa.Column(sa.String(length=20), nullable=True)
+    unit_cost = sa.Column(sa.Numeric(precision=10, scale=5), nullable=True)
     case_size = sa.Column(sa.Numeric(precision=9, scale=4), nullable=True)
+
     regular_price_amount = sa.Column(sa.Numeric(precision=8, scale=3), nullable=True)
     special_order = sa.Column(sa.Boolean(), nullable=True)
     notes = sa.Column(sa.Text(), nullable=True)
