@@ -90,6 +90,7 @@ class ImportHandler(object):
     diff_max_display = 15
     direction = 'import'
     collect_changes_for_processing = True
+    safe_for_web_app = False
 
     def __init__(self, config=None, **kwargs):
         self.config = config
@@ -99,6 +100,34 @@ class ImportHandler(object):
         for key, value in kwargs.items():
             setattr(self, key, value)
 
+    @classmethod
+    def get_key(cls):
+        return 'to_{}.from_{}.{}'.format(cls.local_key, cls.host_key, cls.direction)
+
+    @classmethod
+    def get_spec(cls):
+        return '{}:{}'.format(cls.__module__, cls.__name__)
+
+    @classmethod
+    def get_generic_host_title(cls):
+        if hasattr(cls, 'generic_host_title'):
+            return cls.generic_host_title
+        return "TODO: define `{}.generic_host_title`".format(cls.__name__)
+
+    @classmethod
+    def get_generic_local_title(cls):
+        if hasattr(cls, 'generic_local_title'):
+            return cls.generic_local_title
+        return "TODO: define `{}.generic_local_title`".format(cls.__name__)
+
+    @classmethod
+    def get_generic_title(cls):
+        if hasattr(cls, 'generic_title'):
+            return cls.generic_title
+        host_title = cls.get_generic_host_title()
+        local_title = cls.get_generic_local_title()
+        return "{} -> {}".format(host_title, local_title)
+
     def get_importers(self):
         """
         Returns a dict of all available importers, where the keys are model
@@ -684,6 +713,8 @@ class ToCSVHandler(ToFileHandler):
     """
     Handler for imports which target CSV file(s) on the local side.
     """
+    local_key = 'csv'
+    generic_local_title = "CSV"
 
 
 # TODO: this is still pretty hacky, but needed to get it in here for now
