@@ -610,7 +610,9 @@ class ProductPriceImporter(FromRattail, model.ProductPriceImporter):
 
     @property
     def supported_fields(self):
-        return super(ProductPriceImporter, self).supported_fields + self.product_reference_fields
+        # nb. parent class FromRattail only supports simple_fields, so
+        # we explicitly copy logic from model importer class here.
+        return self.simple_fields + self.product_reference_fields
 
 
 class ProductStoreInfoImporter(FromRattail, model.ProductStoreInfoImporter):
