@@ -374,21 +374,29 @@ class Email(object):
         """
         Returns a list of recipients of the given type for the message.
 
-        :param type_: Must be one of: ``('to', 'cc', 'bcc')``.
+        :param type_: Must be one of: ``('to', 'cc', 'bcc', 'all')``.
+           If you specify ``'all'`` then all recipients will be
+           returned as a single list, with no indication as to which
+           "slot" (to, cc, bcc) each came from.
 
         :rtype: list
         """
         try:
-            if type_.lower() not in ('to', 'cc', 'bcc'):
+            if type_.lower() not in ('to', 'cc', 'bcc', 'all'):
                 raise Exception
         except:
-            raise ValueError("Recipient type must be one of ('to', 'cc', 'bcc'); "
+            raise ValueError("Recipient type must be one of ('to', 'cc', 'bcc', 'all'); "
                              "not: {0}".format(repr(type_)))
         type_ = type_.lower()
-        recips = self.config.getlist('rattail.mail', '{0}.{1}'.format(self.key, type_))
-        if not recips:
-            recips = self.config.getlist('rattail.mail', 'default.{0}'.format(type_))
-        return recips
+        if type_ == 'all':
+            recips = []
+            for typ in ('to', 'cc', 'bcc'):
+                recips.extend(self.get_recips(typ))
+        else:
+            recips = self.config.getlist('rattail.mail', '{0}.{1}'.format(self.key, type_))
+            if not recips:
+                recips = self.config.getlist('rattail.mail', 'default.{0}'.format(type_))
+        return recips or []
 
     def get_prefix(self, data={}, magic=True):
         """
