@@ -62,26 +62,31 @@ class MailMonitorDaemon(Daemon):
         Starts watcher and worker threads according to configuration.
         """
         monitored = load_mailmon_profiles(self.config)
-        for key, profile in six.iteritems(monitored):
 
-            # create a msg queue for the profile
-            profile.queue = queue.Queue()
+        # loop thru all accounts
+        for key, account in six.iteritems(monitored['__accounts__']):
 
-            # create a watcher thread for the IMAP folder
-            watcher = IMAPWatcher(profile, key)
-            name = 'watcher_{}'.format(key)
-            log.info("starting IMAP watcher thread: %s", name)
+            # fire up the account watcher thread
+            watcher = IMAPAccountWatcher(self.config, account)
+            name = 'account_watcher_{}'.format(key)
+            log.info("starting IMAP account watcher thread: %s", name)
             thread = Thread(target=watcher, name=name)
             thread.daemon = True
             thread.start()
 
-            # create an action thread for the profile
-            name = 'actions-{}'.format(key)
-            log.debug("starting action thread: %s", name)
-            thread = Thread(target=perform_actions, name=name, 
-                            args=(self.config, watcher))
-            thread.daemon = True
-            thread.start()
+            # and an action thread for each monitor profile
+            for profile in account.monitored:
+
+                # establish queue for msg transfer between threads
+                profile.queue = queue.Queue()
+
+                # fire up the action thread
+                name = 'actions-{}'.format(profile.key)
+                log.debug("starting action thread: %s", name)
+                thread = Thread(target=perform_actions, name=name,
+                                args=(self.config, watcher, profile))
+                thread.daemon = True
+                thread.start()
 
         # loop indefinitely.  since this is the main thread, the app
         # will terminate when this method ends; all other threads are
@@ -90,92 +95,137 @@ class MailMonitorDaemon(Daemon):
             time.sleep(.01)
 
 
-class IMAPWatcher(object):
+class IMAPAccountWatcher(object):
     """
-    Abstraction to make watching an IMAP folder a little more
-    organized.  Instances of this class are used as callable targets
-    when the daemon starts watcher threads.  They are responsible for
-    polling the IMAP folder and processing any messages found there.
-    """
-    uid_pattern = re.compile(r'^\d+ \(UID (?P<uid>\d+)')
+    Abstraction to make watching an IMAP account a little more
+    organized.
 
-    def __init__(self, profile, key):
-        self.profile = profile
-        self.config = profile.config
-        self.key = key
+    Instances of this class are used as callable targets when the
+    daemon starts watcher threads.  The instance then is responsible
+    for polling the IMAP folder(s) according to the monitor profiles
+    associated with the account.
+    """
+
+    def __init__(self, config, account):
+        self.config = config
+        self.account = account
         self.server = None
-
-        # we need these for writing lastrun times
-        self.lastrun_setting = get_lastrun_setting(self.config, self.key)
+        self.server_recycled = None
+        self.app = config.get_app()
         self.lastrun_timefmt = get_lastrun_timefmt(self.config)
 
-    def get_uid(self, response):
-        match = self.uid_pattern.match(response)
-        if match:
-            return match.group('uid')
+        # assume all profiles should be monitored at first
+        for profile in self.account.monitored:
+            profile.monitor = True
 
     def __call__(self):
         """
-        This is the main loop for the watcher.  It acts as the
+        This is the main loop for the account watcher.  It acts as the
         callable target for the thread in which it runs.  It basically
         checks for new messages, queueing any found, then waits for a
         spell, then does it again, forever.
         """
-        # the 'lastrun' value is maintained as zone-aware UTC
-        lastrun = get_lastrun(self.config, self.key)
-
-        recycled = None
+        lastruns = {}
+        last_error = None
         while True:
-            thisrun = make_utc(tzinfo=True)
+            for profile in self.account.monitored:
 
-            if self.server is None:
-                self.server = imaplib.IMAP4_SSL(self.profile.imap_server)
-                try:
-                    result = self.server.login(self.profile.imap_username, self.profile.imap_password)
-                except self.server.error:
-                    log.exception("failed to login to server!")
-                    return
+                # we may have stopped monitoring some profiles
+                if not profile.monitor:
+                    continue
 
-                log.debug("IMAP server login result: %s", result)
-                recycled = make_utc()
+                # track down last run time for this monitor profile
+                # (nb. run times are maintained as zone-aware UTC)
+                if profile.key in lastruns:
+                    lastrun = lastruns[profile.key]
+                else:
+                    lastrun = get_lastrun(self.config, profile.key)
+                    lastruns[profile.key] = lastrun
 
-                result = self.server.select(self.profile.imap_folder)
-                log.debug("IMAP server select result: %s", result)
+                # check if we should poll this one again yet
+                now = self.app.make_utc(tzinfo=True)
+                if not lastrun or (now - lastrun).seconds >= profile.imap_delay:
 
-            try:
-                self.queue_messages()
-            except:
-                log.exception("failed to queue messages!")
-                if self.profile.stop_on_error:
-                    break
-            else:
-                # successful, so record lastrun time
-                lastrun = thisrun
-                api.save_setting(None, self.lastrun_setting,
-                                 lastrun.strftime(self.lastrun_timefmt))
+                    # yep, so (try to) poll
+                    try:
+                        self.poll_profile(profile)
 
-            # if recycle time limit has been reached, close the IMAP
-            # connection; it will be re-established in next loop run
-            if self.profile.imap_recycle:
-                if (make_utc() - recycled).seconds >= self.profile.imap_recycle:
-                    log.debug("recycle time limit reached, disposing of current connection")
-                    self.server.close()
-                    self.server.logout()
-                    self.server = None
+                    except:
+                        log.exception("failed to poll profile: %s", profile.key)
 
-            # wait a tick before doing it all again
-            time.sleep(self.profile.imap_delay)
+                        # presumably the error has to do with our server
+                        # connection, so make sure we re-connect next time
+                        self.disconnect()
 
-        self.server.close()
-        self.server.logout()
+                        # maybe stop monitoring this profile
+                        if profile.stop_on_error:
+                            profile.monitor = False
+
+                        # skip the rest of this loop; let error delay
+                        # handling kick in below
+                        last_error = now
+                        break
+
+                    else:
+                        # poll went okay, so record last run time
+                        setting = get_lastrun_setting(self.config, profile.key)
+                        api.save_setting(None, setting,
+                                         now.strftime(self.lastrun_timefmt))
+                        lastruns[profile.key] = now
+
+            # maybe pause for a tick on account of error
+            if last_error:
+                if self.account.error_delay:
+                    time.sleep(self.account.error_delay)
+                last_error = None
+
+            # and a general pause just to play nice
+            time.sleep(0.01)
+
+    def connect(self):
+
+        # do we already have a connection?
+        if self.server:
+
+            # keep using forever if no recycle delay
+            if not self.account.recycle_delay:
+                return
+
+            # okay so does it need to be recycled?
+            now = self.app.make_utc()
+            if (now - self.server_recycled).seconds < self.account.recycle_delay:
+
+                # nope it's new enough to keep using
+                return
+
+            # too old, should recycle
+            log.debug("recycle time limit reached (%s seconds), "
+                      "disposing of current connection",
+                      self.account.recycle_delay)
+            self.disconnect()
+
+        # establish new connection
+        self.server = imaplib.IMAP4_SSL(self.account.server)
+        result = self.server.login(self.account.username, self.account.password)
+        log.debug("IMAP server login result: %s", result)
+        self.server_recycled = self.app.make_utc()
+
+    def disconnect(self):
+        try:
+            self.server.close()
+            self.server.logout()
+        except:
+            pass
+
+    def poll_profile(self, profile):
+        self.connect()
+
+        result = self.server.select(profile.imap_folder)
+        log.debug("IMAP server select (%s) result: %s",
+                  profile.imap_folder, result)
 
-    def queue_messages(self):
-        """
-        Check for new messages in the folder, and queue any found, for
-        action processing thread.
-        """
         # maybe look for "all" or maybe just "unread"
-        if self.profile.imap_unread_only:
+        if profile.imap_unread_only:
             criterion = '(UNSEEN)'
         else:
             criterion = 'ALL'
@@ -188,22 +238,21 @@ class IMAPWatcher(object):
         # config may dictacte a "max batch size" in which case we will
         # only queue so many messages at a time
         uids = items[0].split()
-        if self.profile.max_batch_size:
-            if len(uids) > self.profile.max_batch_size:
-                uids = uids[:self.profile.max_batch_size]
+        if profile.max_batch_size:
+            if len(uids) > profile.max_batch_size:
+                uids = uids[:profile.max_batch_size]
 
         # add message uids to the queue
         for uid in uids:
-            self.profile.queue.put(uid)
+            profile.queue.put(uid)
 
 
-def perform_actions(config, watcher):
+def perform_actions(config, watcher, profile):
     """
     Target for action threads.  Provides the main loop which checks
     the queue for new messages and invokes actions for each, as they
     appear.
     """
-    profile = watcher.profile
     stop = False
     while not stop:
 
