@@ -252,6 +252,21 @@ class PeopleHandler(GenericHandler):
         """
         address.invalid = invalid
 
+    def resolve_person(self, pending, person, user, **kwargs):
+        """
+        Resolve a pending person.
+
+        :param pending: Reference to a PendingCustomer instance.
+
+        :param person: Reference to a Person instance.
+
+        :param user: Reference to the User responsible.
+        """
+        custorder_handler = self.app.get_custorder_handler()
+        custorder_handler.resolve_person(pending, person, user)
+
+        pending.status_code = self.enum.PENDING_CUSTOMER_STATUS_RESOLVED
+
     def request_merge(self, user, removing_uuid, keeping_uuid, **kwargs):
         """
         Submit an officical merge request for two Person records.
