@@ -31,6 +31,7 @@ import sqlalchemy as sa
 from sqlalchemy import orm
 
 from rattail import pod
+from rattail.db import api
 from rattail.util import load_object
 from rattail.app import GenericHandler
 from rattail.gpc import GPC
@@ -46,14 +47,60 @@ class ProductsHandler(GenericHandler):
     particular product can be deleted, etc.
     """
 
+    def make_gpc(self, value, **kwargs):
+        """
+        Try to convert the given value to a :class:`~rattail.gpc.GPC`
+        instance, and return the result.
+        """
+        return GPC(value, **kwargs)
+
+    def make_full_description(self, product=None,
+                              brand_name=None, description=None, size=None,
+                              **kwargs):
+        """
+        Return a "full" description for the given product, or
+        attributes thereof.
+
+        :param product: Optional, but can be a reference to either a
+           :class:`~rattail.db.model.products.Product` or
+           :class:`~rattail.db.model.products.PendingProduct`
+           instance.
+
+        :param brand_name: Optional; brand name as string.  If not
+           provided then will be obtained from ``product`` param.
+
+        :param description: Optional; description as string.  If not
+           provided then will be obtained from ``product`` param.
+
+        :param size: Optional; size as string.  If not provided then
+           will be obtained from ``product`` param.
+        """
+        from rattail.db.util import make_full_description
+
+        model = self.model
+
+        if brand_name is None and product:
+            if product.brand:
+                brand_name = product.brand.name
+            elif isinstance(product, model.PendingProduct):
+                brand_name = product.brand_name
+
+        if description is None and product:
+            description = product.description
+
+        if size is None and product:
+            size = product.size
+
+        return make_full_description(brand_name, description, size)
+
     def find_products_by_key(self, session, value, **kwargs):
         """
         Locate any products where the "key" matches the given value.
 
         By default this search is as "thorough" as possible and may
         return multiple results in some cases where you might not
-        expect them.  Please pass the :param:`only` param if you need
-        a more focused search etc.
+        expect them.  Please pass the ``only`` param if you need a
+        more focused search etc.
 
         :param session: Session for the Rattail database.
 
@@ -75,11 +122,10 @@ class ProductsHandler(GenericHandler):
            return value include indications of which key was matched,
            for each product result.
 
-        :returns: If :param:`include_keys` is true, then will return a
-           list of 2-tuples representing each match.  The first
-           element of the tuple will be the "key" field pseudonym
-           which was matched on; the second element will be the
-           product.
+        :returns: If ``include_keys`` is true, then will return a list
+           of 2-tuples representing each match.  The first element of
+           the tuple will be the "key" field pseudonym which was
+           matched on; the second element will be the product.
 
            But if ``include_keys`` is false, then will return just a
            simple list of the products, i.e. won't include the keys.
@@ -180,6 +226,122 @@ class ProductsHandler(GenericHandler):
 
         return products
 
+    def locate_product_for_entry(self, session, entry, **kwargs):
+        """
+        This method aims to provide sane default logic for locating a
+        :class:`~rattail.db.model.products.Product` record for the
+        given "entry" value.
+
+        The default logic here will try to honor the "configured"
+        product key field, and prefer that when attempting the lookup.
+
+        :param session: Reference to current DB session.
+
+        :param entry: Value to use for lookup.  This is most often a
+           simple string, but the method can handle a few others.  For
+           instance it is common to read values from a spreadsheet,
+           and sometimes those come through as integers etc.  If this
+           value is a :class:`~rattail.gpc.GPC` instance, special
+           logic may be used for the lookup.
+        """
+        # don't bother if we're given empty "entry" value
+        if not entry:
+            return
+
+        # if entry is GPC then only look for that type of match
+        if isinstance(entry, GPC):
+            return self.locate_product_for_gpc(session, entry)
+
+        # for the rest of this, assume entry is string
+        entry = six.text_type(entry)
+        model = self.model
+
+        # try to locate product by uuid before other, more specific key
+        # if kwargs.get('try_uuid', True):
+        product = session.query(model.Product).get(entry)
+        if product:
+            return product
+
+        # prefer caller-provided key over configured key
+        product_key = kwargs.get('product_key')
+        if not product_key:
+            product_key = self.config.product_key()
+
+        if product_key == 'upc':
+
+            if entry.isdigit():
+
+                # we first assume the user entry *does* include check digit
+                provided = GPC(entry, calc_check_digit=False)
+                product = api.get_product_by_upc(session, provided)
+                if product:
+                    return product
+
+                # but we can also calculate a check digit and try that
+                checked = GPC(entry, calc_check_digit='upc')
+                product = api.get_product_by_upc(session, checked)
+                if product:
+                    return product
+
+                # one last trick is to expand UPC-E to UPC-A and then reattempt
+                # the lookup, *with* check digit (since it would be known)
+                if len(entry) in (6, 8):
+                    checked = GPC(upce_to_upca(entry), calc_check_digit='upc')
+                    product = api.get_product_by_upc(session, checked)
+                    if product:
+                        return product
+
+        elif product_key == 'item_id':
+
+            # try to locate product by item_id
+            product = api.get_product_by_item_id(session, entry)
+            if product:
+                return product
+
+        elif product_key == 'scancode':
+
+            # try to locate product by scancode
+            product = api.get_product_by_scancode(session, entry)
+            if product:
+                return product
+
+        # if we made it this far, lookup by product key failed.
+
+        # try a vendor item code lookup, only if caller requests it
+        if kwargs.get('lookup_vendor_code'):
+            product = api.get_product_by_vendor_code(session, entry,
+                                                     vendor=kwargs.get('vendor'))
+            if product:
+                return product
+
+        # try an "alternate code" lookup, only if caller requests it
+        if kwargs.get('lookup_by_code'):
+            product = api.get_product_by_code(session, entry)
+            if product:
+                return product
+
+    def locate_product_for_gpc(self, session, gpc, **kwargs):
+        """
+        Try to locate a product for the given ``GPC`` value.
+        """
+        if not gpc:
+            return
+
+        # first we try the normal search
+        product = api.get_product_by_upc(session, gpc)
+        if product:
+            return product
+
+        # maybe also try special search for "Type 2 UPC"
+        if gpc.type2_upc and self.convert_type2_for_gpc_lookup():
+            cleaned = GPC('002{}00000'.format(gpc.data_str[1:6]),
+                          calc_check_digit='upc')
+            return api.get_product_by_upc(session, cleaned)
+
+    def convert_type2_for_gpc_lookup(self):
+        return self.config.getbool('rattail', 'products.convert_type2_for_gpc_lookup',
+                                   default=False)
+
     def get_url(self, product, **kwargs):
         """
         Return the Tailbone "view" URL for the given product.
@@ -252,6 +414,32 @@ class ProductsHandler(GenericHandler):
                 self.app.render_currency(price.pack_price),
                 price.pack_multiple)
 
+    def make_pending_product(self, **kwargs):
+        """
+        Create and return a new
+        :class:`~rattail.db.model.products.PendingProduct` instance,
+        per the given kwargs.
+        """
+        model = self.model
+        kwargs.setdefault('status_code', self.enum.PENDING_PRODUCT_STATUS_PENDING)
+        pending = model.PendingProduct(**kwargs)
+        return pending
+
+    def resolve_product(self, pending, product, user, **kwargs):
+        """
+        Resolve a pending product.
+
+        :param pending: Reference to a PendingProduct instance.
+
+        :param product: Reference to a Product instance.
+
+        :param user: Reference to the User responsible.
+        """
+        custorder_handler = self.app.get_custorder_handler()
+        custorder_handler.resolve_product(pending, product, user)
+
+        pending.status_code = self.enum.PENDING_PRODUCT_STATUS_RESOLVED
+
     def get_uom_sil_codes(self, session, uppercase=False, **kwargs):
         """
         This should return a dict, keys of which are UOM abbreviation strings,
