@@ -41,6 +41,8 @@ class FromTrainwreckHandler(FromSQLAlchemyHandler):
     """
     Base class for import handlers which have a Trainwreck DB as data source on the host side.
     """
+    host_key = 'trainwreck'
+    generic_host_title = "Trainwreck"
     host_title = "Trainwreck"
 
     def make_host_session(self):
@@ -51,6 +53,8 @@ class ToTrainwreckHandler(ToSQLAlchemyHandler):
     """
     Base class for import handlers which target a Trainwreck DB on the local side.
     """
+    local_key = 'trainwreck'
+    generic_local_title = "Trainwreck"
     local_title = "Trainwreck"
 
     def make_session(self):
